@@ -16,7 +16,7 @@ from typing import overload
 from typing import TypeAlias
 from typing import TypeVar
 
-from . import utils
+from .utils.common import get_parent_doc
 
 
 class Event:
@@ -26,19 +26,19 @@ class Event:
 
         Event() >> EventBus['foo']
     """
-    __slots__ = ()
+    __slots__: tuple[str, ...] = ()
 
     def __repr__(self) -> str:
-        values = {attr: val for attr, val in ((attr, getattr(self, attr)) for attr in self.__slots__)}
+        values = {attr: getattr(self, attr) for attr in self.__slots__}
         return f'<"{self.name}" Event {values=}>' if self.__slots__ else f'<Empty {self.name}>'
 
     def __rshift__(self, __bus: EventBus, /) -> None:
         """Syntax sugar for __bus.fire(event)"""
-        __bus.fire(event=self)
+        return __bus.fire(event=self)
 
     def __rlshift__(self, __bus: EventBus, /) -> None:
         """Right shift if the EventBus.__lshift__ dunder is not working."""
-        self >> __bus
+        return self >> __bus
 
     @property
     def name(self) -> str:
@@ -54,10 +54,10 @@ class Event:
 
         Defaults to the first line of the nearest type doc in the mro.
         """
-        doc = self.__doc__
+        doc: str | None = self.__doc__
         if doc is None and isinstance(self, Event) and type(self) is not Event:
-            doc = utils.get_parent_doc(type(self))
-        return doc.splitlines()[0]
+            doc = get_parent_doc(type(self))
+        return doc.splitlines()[0] if doc is not None else ''
 
 
 _ET_co = TypeVar('_ET_co', bound=Event, covariant=True)  # Bound to Event. Can use Event subclass instances in place of Event instances.
@@ -79,7 +79,7 @@ class _Subscribers(defaultdict[type[Event], list[tuple[
         repr_: str = ''
         for event in self:
             repr_ += f'{event.__name__}[{len(self[event])}], '
-        return f'({repr_.rstrip()})'
+        return f'({repr_.rstrip(", ")})'
 
     def add(self, event: type[_ET_co], callable_pair: tuple[_EventRunnable, _EventPredicate | None]) -> None:
         """Add a callable pair to an Event subscriber list."""
@@ -103,11 +103,14 @@ class _EventBusMeta(type):
 
     @classmethod
     @overload
-    def __getitem__(mcs, _: type[Event] | str, /) -> EventBus | None:
-        """Exist to support Generic Typing on :py:class:`EventBus`."""
+    def __getitem__(mcs, id: type[Event]) -> None: ...
 
     @classmethod
-    def __getitem__(mcs, id: str) -> EventBus | None:
+    @overload
+    def __getitem__(mcs, id: str) -> EventBus | None: ...
+
+    @classmethod
+    def __getitem__(mcs, id):
         return mcs._id_bus_map.get(id.lower())
 
     @classmethod
@@ -161,17 +164,18 @@ class EventBus(Generic[_ET_co], metaclass=_EventBusMeta):
         :param __id: id to register this instance as.
         :raises KeyError: When a bus with the given id already exists.
         """
-        if __id is None or type(self)[__id] is None:
-            self.id = __id
-            self._subscribers = _Subscribers()
-            if __id is not None:
-                type(self)[__id] = self
-        else:
+        if __id is not None and type(self)[__id] is not None:
             raise KeyError(f'EventBus id "{__id}" is already registered in {type(type(self)).__name__}')
 
+        self.id: str | None = __id
+        self._subscribers: _Subscribers = _Subscribers()
+
+        if __id is not None:
+            type(self)[__id] = self
+
     def __lshift__(self, __event: _ET_co | type[_ET_co], /) -> None:
         """Syntax sugar for self.fire(event)"""
-        self.fire(__event)
+        return self.fire(__event)
 
     def __repr__(self) -> str:
         return f'<{type(self).__name__} id={self.id!r}; Subscribers={self._subscribers!r}>'
@@ -212,6 +216,7 @@ class EventBus(Generic[_ET_co], metaclass=_EventBusMeta):
                         # Finally, call
                         e_callable_pair[0](event)
 
+    # pylint: disable=useless-param-doc
     def subscribe(self, __callable: _EventRunnable, /, event: type[_ET_co], event_predicate: _EventPredicate | None = None) -> None:
         """Subscribe a :py:class:`Callable` to an :py:class:`Event` type.
 
