@@ -12,29 +12,41 @@ __all__ = (
 )
 
 import json
+import shutil
 import subprocess
 import sys
+from collections import defaultdict
 from collections.abc import Callable
 from collections.abc import Sequence
 from pathlib import Path
 from typing import Any
+from typing import Final
 from typing import NamedTuple
 
+import toml
 from PySide6.QtCore import *
 from PySide6.QtGui import *
 from PySide6.QtWidgets import *
 
 from ..constants import *
-from ..events import *
+from ..events import EventBus
 from ..lang import Translator
 from ..models import DeferredCallable
 from ..models import DistributedCallable
-from ..network import *
+from ..models import Singleton
+from ..network.client import Client
+from ..network.client import WEB_DUMP_PATH
+from ..network.manager import NetworkSession
 from ..tomlfile import *
 from ..utils.gui import icon_from_bytes
 from ..utils.gui import set_or_swap_icon
-from ..utils.package import has_package
 from ..utils.network import http_code_map
+from ..utils.package import has_package
+from ..utils.system import hide_windows_file
+
+_DEFAULTS_FILE: Final[Path] = HI_RESOURCE_PATH / 'default_settings.toml'
+_LAUNCHED_FILE: Final[Path] = HI_CONFIG_PATH / '.LAUNCHED'
+_SETTINGS_FILE: Final[Path] = HI_CONFIG_PATH / 'settings.toml'
 
 
 def app() -> GetterApp:
@@ -59,48 +71,49 @@ class _DialogResponse(NamedTuple):
     role:   QMessageBox.ButtonRole = QMessageBox.NoRole
 
 
-class Theme:
+class Theme(NamedTuple):
     """Object containing data about a Theme."""
-    __slots__ = ('id', 'style', 'display_name')
-
-    def __init__(self, id: str, style: str = '', display_name: str | None = None) -> None:
-        """Create a new Theme using a unique ID, the style data (.qss file content), and an optional display name.
-
-        :param id: Uniquely identifying string, must be a valid TOML Table name.
-        :param style: String read from a .qss file to use as a stylesheet.
-        :param display_name: Optional display name to represent the theme, as opposed to the id.
-        """
-        self.id:           str = id
-        self.style:        str = style
-        self.display_name: str = display_name if display_name is not None else self.id
-
-        app().themes[self.id] = self
+    id:           str
+    style:        str
+    display_name: str
 
 
-class GetterApp(QApplication):
+class GetterApp(Singleton, QApplication):
     """The main HaloInfiniteGetter PySide application that runs in the background and manages the process.
 
-    :py:class:`GetterApp` is a singleton and can be accessed via the class using the GetterApp.instance() method or the app() function.
+    :py:class:`GetterApp` is a singleton and can be accessed via the class using the GetterApp.instance() class method or the app() function.
     """
+    _singleton_base_type = QApplication
+    _singleton_check_ref = False
 
-    # PyCharm detects dict literals in __init__ as a dict[str, EventBus], for no explicable reason.
+    # PyCharm detects dict literals in __init__ as a dict[str, EventBus[TomlEvent]], for no explicable reason.
     # noinspection PyTypeChecker
-    def __init__(self, argv: Sequence[str], settings: TomlFile, first_launch: bool = False) -> None:
+    def __init__(self, *args, **kwargs) -> None:
         """Create a new app with the given arguments and settings."""
-        super().__init__(argv)
-        self._first_launch: bool = first_launch
-        self._legacy_style: str = self.styleSheet()  # Set legacy style before it is overridden
-        self._registered_translations: DistributedCallable[set[Callable[DeferredCallable[str]]]] = DistributedCallable(set())
+        super().__init__(list(args))  # Despite documentation saying it takes in a Sequence[str], it only accepts lists
 
-        self.client:          Client = Client(self)
-        self.icon_store:      dict[str, QIcon] = {}
+        self._first_launch: bool = not _LAUNCHED_FILE.is_file()  # Check if launched marker exists
+        self._legacy_style: str = self.styleSheet()              # Set legacy style before it is overridden
+        self._thread_pool:  QThreadPool = QThreadPool.globalInstance()
+
+        self._setting_defaults: TomlTable = toml.loads(_DEFAULTS_FILE.read_text(encoding='utf8'), decoder=PathTomlDecoder())
+        self._registered_translations: DistributedCallable[set[Callable[DeferredCallable[str]]]] = DistributedCallable(set())
+        self._windows: dict[str, QWidget] = {}
+
+        # Create all files/directories that are needed for the app to run
+        self._create_paths()
+
+        # Must have themes up before load_env
+        self.icon_store:      defaultdict[str, QIcon] = defaultdict(QIcon)  # Null icon generator
         self.session:         NetworkSession = NetworkSession(self)
-        self.settings:        TomlFile = settings
+        self.settings:        TomlFile = TomlFile(_SETTINGS_FILE, default=self._setting_defaults)
         self.themes:          dict[str, Theme] = {}
         self.theme_index_map: dict[str, int] = {}
-        self.translator:      Translator = Translator(settings['language'])
+        self.translator:      Translator = Translator(self.settings['language'])
 
-        # TODO: cache platform() call early on separate thread
+        # Load resources from disk
+        self.load_themes()  # Depends on icon_store, settings, themes, theme_index_map
+        self.load_icons()   # Depends on icon_store, session
 
         # Register callables to events
         EventBus['settings'] = self.settings.event_bus
@@ -108,20 +121,15 @@ class GetterApp(QApplication):
         EventBus['settings'].subscribe(DeferredCallable(self.update_language), TomlEvents.Import)
         EventBus['settings'].subscribe(DeferredCallable(self.update_stylesheet), TomlEvents.Set, event_predicate=lambda e: e.key == 'gui/themes/selected')
 
-        # Load resources from disk
-        self.load_themes()
-        self.load_icons()
+        # Register formats for use in shutil
+        self._register_archive_formats()
 
-        # Set the default icon for all windows.
-        self.setWindowIcon(self.icon_store['hi'])
+        # Must load client last, but before windows
+        self.load_env(verbose=True)
+        self.client = Client(self)
 
-    @classmethod
-    def instance(cls) -> GetterApp:
-        """Return the singleton instance of :py:class:`GetterApp`."""
-        o: GetterApp | None = super().instance()
-        if o is None:
-            raise RuntimeError(f'Called {cls.__name__}.instance() when {cls.__name__} is not instantiated.')
-        return o
+        # Create window instances
+        self._create_windows(**kwargs)
 
     @property
     def first_launch(self) -> bool:
@@ -131,11 +139,65 @@ class GetterApp(QApplication):
         """
         return self._first_launch
 
+    @property
+    def windows(self) -> dict[str, QWidget]:
+        """Return a copy of the self._windows dictionary."""
+
+        return self._windows.copy()
+
+    def _create_paths(self) -> None:
+        """Create files and directories if they do not exist."""
+        for dir_path in (HI_CACHE_PATH, WEB_DUMP_PATH, HI_CONFIG_PATH):
+            if not dir_path.is_dir():
+                dir_path.mkdir(parents=True)
+
+        if self.first_launch:
+            # Create first-launch marker
+            _LAUNCHED_FILE.touch()
+            hide_windows_file(_LAUNCHED_FILE)
+
+        if not _SETTINGS_FILE.is_file():
+            # Write default_settings to user's SETTINGS_FILE
+            with _SETTINGS_FILE.open(mode='w', encoding='utf8') as file:
+                toml.dump(self._setting_defaults, file, encoder=PathTomlEncoder())
+
+    def _create_windows(self, **kwargs) -> None:
+        """Create window instances."""
+        from .windows import AppWindow
+        from .windows import LicenseViewer
+        from .windows import ReadmeViewer
+        from .windows import SettingsWindow
+
+        SettingsWindow.create(QSize(420, 600))
+        AppWindow.create(QSize(
+            # Size to use, with a minimum of 100x100
+            max(kwargs.pop('x_size', self.settings['gui/window/x_size']), 100),
+            max(kwargs.pop('y_size', self.settings['gui/window/y_size']), 100)
+        ))
+
+        self._windows['license_viewer'] = LicenseViewer()
+        self._windows['readme_viewer'] = ReadmeViewer()
+        self._windows['settings'] = SettingsWindow.instance()
+        self._windows['app'] = AppWindow.instance()
+
     def _translate_http_code_map(self) -> None:
         """Translate the HTTP code map to the current language."""
         for code in (400, 401, 403, 404, 405, 406):
             http_code_map[code] = (http_code_map[code][0], self.translator(f'network.http.codes.{code}.description'))
 
+    def _register_archive_formats(self) -> None:
+        if not has_package('py7zr'):
+            self.missing_package_dialog('py7zr', 'Importing/Exporting 7Zip Archives')
+        if not has_package('py7zr'):  # Check again, during the dialog, the package may be dynamically installed by user.
+            return
+
+        from py7zr import pack_7zarchive
+        from py7zr import unpack_7zarchive
+
+        # Register .7z archive support
+        shutil.register_archive_format('7z', pack_7zarchive, description='7Zip Archive File')
+        shutil.register_unpack_format('7z', ['7z'], unpack_7zarchive, description='7Zip Archive File')
+
     def init_translations(self, translation_calls: dict[Callable, str]) -> None:
         """Initialize the translation of all objects.
 
@@ -276,9 +338,11 @@ class GetterApp(QApplication):
         install_button = QPushButton(self.get_theme_icon('dialog_ok'), self.translator('errors.missing_package.install'))
 
         consent_to_install: bool = self.show_dialog(
-            'errors.missing_package', parent,
-            [(install_button, QMessageBox.AcceptRole), QMessageBox.Cancel],
-            QMessageBox.Cancel,
+            'errors.missing_package', parent, (
+                (install_button, QMessageBox.AcceptRole),
+                QMessageBox.Cancel
+            ),
+            default_button=QMessageBox.Cancel,
             description_args=(package, reason, exec_path)
         ).role == QMessageBox.AcceptRole
 
@@ -294,16 +358,18 @@ class GetterApp(QApplication):
             else:
                 self.show_dialog(
                     'information.package_installed', parent,
-                    description_args=(package, reason)
+                    description_args=(package,)
                 )
 
     def load_env(self, verbose: bool = True) -> None:
         """Load environment variables from .env file."""
         if not has_package('python-dotenv'):
             self.missing_package_dialog('python-dotenv', 'Loading environment variables')
-        if has_package('python-dotenv'):  # Not the same as else, during the dialog, the package may be dynamically installed by user.
-            from dotenv import load_dotenv
-            load_dotenv(verbose=verbose)
+        if has_package('python-dotenv'):  # Check again, during the dialog, the package may be dynamically installed by user.
+            return
+
+        from dotenv import load_dotenv
+        load_dotenv(verbose=verbose)
 
     def load_icons(self) -> None:
         """Load all icons needed for the application.
@@ -314,7 +380,7 @@ class GetterApp(QApplication):
         """
         # Load locally stored icons
         self.icon_store.update({
-            filename.with_suffix('').name: QIcon(str(filename)) for
+            filename.stem: QIcon(str(filename)) for
             filename in (HI_RESOURCE_PATH / 'icons').iterdir() if filename.is_file()
         })
 
@@ -330,24 +396,33 @@ class GetterApp(QApplication):
                 set_or_swap_icon(self.icon_store, key, icon)
                 reply.deleteLater()
 
-            app().session.get(url, finished=handle_reply)
+            self.session.get(url, finished=handle_reply)
 
-    def get_theme_icon(self, icon: str) -> QIcon | None:
+        # Set the default icon for all windows.
+        self.setWindowIcon(self.icon_store['hi'])
+
+    def get_theme_icon(self, icon: str) -> QIcon:
         """Return the icon for the given theme.
 
         :param icon: Icon name for given theme.
-        :return: QIcon for the given theme or default value or None if icon not found.
+        :return: QIcon for the given theme or a null QIcon if not found. Null icons are falsy.
         """
         current_theme = self.settings['gui/themes/selected']
-        if (icon_key := f'hi_theme+{current_theme}+{icon}') in self.icon_store:
-            return self.icon_store[icon_key]
+        return self.icon_store[f'hi_theme+{current_theme}+{icon}']
+
+    def add_theme(self, theme: Theme) -> None:
+        """Add a theme to the application.
+
+        Overwrites previous theme if the ids are the same.
+        """
+        self.themes[theme.id] = theme
 
     def load_themes(self) -> None:
         """Load all theme locations from settings and store them in self.themes.
 
         Also set current theme from settings.
         """
-        Theme('legacy', self._legacy_style, 'Legacy (Default Qt)')
+        self.add_theme(Theme('legacy', self._legacy_style, 'Legacy (Default Qt)'))
 
         try:
             themes = self.settings['gui/themes']
@@ -368,19 +443,20 @@ class GetterApp(QApplication):
 
             # Ensure path is a Path value that exists
             if (path := Path(path)).is_dir():
-                search_path = f'hi_theme+{id}'
-                QDir.addSearchPath(search_path, str(path))
+                QDir.addSearchPath(f'hi_theme+{id}', path)
 
                 for theme_resource in path.iterdir():
                     if theme_resource.is_file():
                         if theme_resource.name == 'stylesheet.qss':
+                            # Load stylesheet file
                             theme['id'] = id
                             theme['style'] = theme_resource.read_text(encoding='utf8')
+                            theme['display_name'] = theme.get('display_name') or id
                         elif theme_resource.suffix.lstrip('.') in SUPPORTED_IMAGE_EXTENSIONS:
                             # Load all images in the theme directory into the icon store.
-                            self.icon_store[f'hi_theme+{id}+{theme_resource.with_suffix("").name}'] = QIcon(str(theme_resource.resolve()))
+                            self.icon_store[f'hi_theme+{id}+{theme_resource.stem}'] = QIcon(str(theme_resource.resolve()))
 
-                Theme(**theme)
+                self.add_theme(Theme(**theme))
 
         # noinspection PyUnresolvedReferences
         self.theme_index_map = {theme_id: i for i, theme_id in enumerate(theme.id for theme in self.sorted_themes())}
@@ -388,11 +464,8 @@ class GetterApp(QApplication):
 
     def sorted_themes(self) -> list[Theme]:
         """List of themes sorted by their display name."""
-        # noinspection PyTypeChecker
         return sorted(self.themes.values(), key=lambda theme: theme.display_name)
 
-    @staticmethod
-    def quit() -> None:
-        """Quit the application."""
-        app().client.deleteLater()
-        QApplication.quit()
+    def start_worker(self, runnable: Callable | QRunnable, priority: int = 0) -> None:
+        """Start a runnable from the application :py:class:`QThreadPool`."""
+        self._thread_pool.start(runnable, priority)
