@@ -4,11 +4,15 @@
 """Package containing :py:class:`QMenu` Context Menus."""
 
 __all__ = (
+    'CacheIndexContextMenu',
+    'ColumnContextMenu',
     'FileContextMenu',
     'HelpContextMenu',
     'ToolsContextMenu',
 )
 
+from .cache_index import CacheIndexContextMenu
+from .column import ColumnContextMenu
 from .file import FileContextMenu
 from .help import HelpContextMenu
 from .tools import ToolsContextMenu
