@@ -6,6 +6,7 @@ from __future__ import annotations
 
 __all__ = (
     'ExceptionLogger',
+    'LoggedException',
 )
 
 from datetime import datetime
@@ -15,19 +16,10 @@ from typing import NamedTuple
 from PySide6.QtWidgets import *
 
 from ...events import EventBus
-from ...exceptions import ExceptionEvent
+from ...exception_hook import ExceptionEvent
 from ..app import tr
 
 
-class _LoggedException(NamedTuple):
-    """Container for a logged exception. Includes the severity of the exception, the exception itself, an optional traceback, and a timestamp."""
-    severity:  int | None = None
-    exception: Exception | None = None
-    traceback: TracebackType | None = None
-    timestamp: datetime | None = None
-
-
-# noinspection PyArgumentList
 class ExceptionLogger(QPushButton):
     """A :py:class:`QPushButton` that logs exceptions to the event bus."""
 
@@ -45,7 +37,7 @@ class ExceptionLogger(QPushButton):
         EventBus['exceptions'].subscribe(self.on_exception, ExceptionEvent)
 
         self.label:          QLabel = QLabel(self)
-        self.exception_log:  list[_LoggedException] = []
+        self.exception_log:  list[LoggedException] = []
         self.reporter:       ExceptionReporter = ExceptionReporter(self)
         self.severity:       int = 0
 
@@ -76,10 +68,10 @@ class ExceptionLogger(QPushButton):
             level = 1
         else:
             level = 2
-        if self.severity < level:
-            self.severity = level
 
-        self.exception_log.append(_LoggedException(level, event.exception, event.traceback, datetime.now()))
+        self.severity = max(self.severity, level)
+
+        self.exception_log.append(LoggedException(level, event.exception, event.traceback, datetime.now()))
         self.sort_exceptions()
 
         logged = len(self.exception_log)
@@ -104,3 +96,11 @@ class ExceptionLogger(QPushButton):
         self._severity = value
         self.setIcon(self.style().standardIcon(self.level_icon_list[self.severity]))
         self.reporter.setWindowIcon(self.icon())
+
+
+class LoggedException(NamedTuple):
+    """Container for a logged exception. Includes the severity of the exception, the exception itself, an optional traceback, and a timestamp."""
+    severity:  int | None = None
+    exception: Exception | None = None
+    traceback: TracebackType | None = None
+    timestamp: datetime | None = None
