@@ -6,13 +6,14 @@ from __future__ import annotations
 
 __all__ = (
     'AppWindow',
+    'size_label_for',
 )
 
 import json
 import string
 import sys
-import webbrowser
 from collections.abc import Callable
+from pathlib import Path
 from typing import Any
 
 from PySide6.QtCore import *
@@ -22,9 +23,12 @@ from PySide6.QtWidgets import *
 from ..._version import __version__
 from ...constants import *
 from ...events import EventBus
-from ...exceptions import ExceptionEvent
+from ...exception_hook import ExceptionEvent
 from ...models import DeferredCallable
 from ...models import DistributedCallable
+from ...models import Singleton
+from ...tomlfile import TomlEvents
+from ...utils.gui import init_layouts
 from ...utils.gui import init_objects
 from ...utils.gui import scroll_to_top
 from ...utils.network import decode_url
@@ -33,20 +37,32 @@ from ..app import app
 from ..app import tr
 from ..menus import *
 from ..widgets import *
+from ..workers import RecursiveSearch
 from .exception_reporter import ExceptionReporter
-from .readme_viewer import ReadmeViewer
 
 
-# noinspection PyArgumentList
-class AppWindow(QMainWindow):
+def size_label_for(num: int) -> str:
+    """Return the best display unit to describe the given data's size.
+
+    Ex: Bytes, KiB, MiB, GiB, TiB
+    """
+    display_unit = 'Bytes'
+    for size_label, size in BYTE_UNITS.items():
+        if num >= (size // 2):
+            display_unit = size_label
+        else:
+            break
+    return display_unit
+
+
+class AppWindow(Singleton, QMainWindow):
     """Main window for the HaloInfiniteGetter application."""
-
+    _singleton_base_type = QMainWindow
+    _singleton_check_ref = False
     shown_key_warning: bool = False
 
     def __init__(self, size: QSize) -> None:
         """Create the window for the application."""
-        from .settings import SettingsWindow
-
         super().__init__()
         app().client.receivedError.connect(self.update_error)
         app().client.receivedData.connect(self.update_image)
@@ -57,7 +73,10 @@ class AppWindow(QMainWindow):
         self.change_title(tr('app.name') + f' v{__version__}')
         self.resize(size)
 
-        self.settings_window = SettingsWindow(self, QSize(420, 600))
+        for subscribe_params in (
+                (DeferredCallable(self.resize_image), TomlEvents.Set, lambda event: event.key.startswith('gui/media_output/')),
+                (lambda val: self.text_output.setLineWrapMode(val.new), TomlEvents.Set, lambda event: event.key == 'gui/text_output/line_wrap_mode'),
+        ): EventBus['settings'].subscribe(*subscribe_params)
 
         self.exception_reporter:  ExceptionReporter
         self.input_field:         HistoryComboBox
@@ -79,43 +98,48 @@ class AppWindow(QMainWindow):
 
     def _init_toolbar(self) -> None:
         """Initialize toolbar widgets."""
+        from .settings import SettingsWindow
 
         def context_menu_handler(menu_class: type[QMenu]) -> None:
             """Create a new :py:class:`QMenu` and show it at the cursor's position."""
             if not issubclass(menu_class, QMenu):
                 raise TypeError(f'{menu_class} is not a subclass of {QMenu}')
 
-            menu = menu_class(self)
-            menu.setAttribute(Qt.WA_DeleteOnClose)
-
-            menu.move(self.cursor().pos())
-            menu.show()
+            menu: QMenu = menu_class(self)
+            menu.exec(self.cursor().pos())
+            menu.deleteLater()
 
         init_objects({
             (menu_bar := QToolBar(self)): {},
+
             (status_bar := QToolBar(self)): {
                 'movable': False,
             },
+
             (file := QAction(self)): {
                 'menuRole': QAction.MenuRole.ApplicationSpecificRole,
                 'triggered': DeferredCallable(context_menu_handler, FileContextMenu)
             },
+
             (settings := QAction(self)): {
                 'menuRole': QAction.MenuRole.PreferencesRole,
                 'triggered': DistributedCallable((
-                    self.settings_window.show,
-                    self.settings_window.activateWindow,
-                    self.settings_window.raise_
+                    SettingsWindow.instance().show,
+                    SettingsWindow.instance().activateWindow,
+                    SettingsWindow.instance().raise_
                 ))
             },
+
             (tools := QAction(self)): {
                 'menuRole': QAction.MenuRole.ApplicationSpecificRole,
                 'triggered': DeferredCallable(context_menu_handler, ToolsContextMenu)
             },
+
             (help := QAction(self)): {
                 'menuRole': QAction.MenuRole.AboutRole,
                 'triggered': DeferredCallable(context_menu_handler, HelpContextMenu)
             },
+
             (logger := ExceptionLogger(self)): {
                 'size': {'fixed': (None, 20)},
                 'clicked': DistributedCallable((
@@ -124,7 +148,7 @@ class AppWindow(QMainWindow):
                     logger.reporter.raise_
                 ))
             }
-        }, translator=tr)
+        })
 
         app().init_translations({
             menu_bar.setWindowTitle: 'gui.menu_bar.title',
@@ -150,6 +174,7 @@ class AppWindow(QMainWindow):
         status_bar.addSeparator()
         status_bar.addWidget(logger.label)
 
+    # noinspection PyTypeChecker
     def _init_ui(self) -> None:
         """Initialize the UI, including Layouts and widgets."""
 
@@ -161,11 +186,14 @@ class AppWindow(QMainWindow):
             :param handler: Callable to execute when closed, to reattach the frame to the parent window.
             :param title: The window title.
             """
-            window = QMainWindow()
-            window.setWindowTitle(title if title is not None else self.windowTitle())
-            window.setCentralWidget(frame)
-            window.setMinimumHeight(200)
-            window.setMinimumWidth(300)
+            init_objects({
+                (window := QMainWindow()): {
+                    'windowTitle': title if title is not None else self.windowTitle(),
+                    'centralWidget': frame,
+                    'size': {'minimum': (300, 200)}
+                }
+            })
+
             window.closeEvent = lambda *_: handler() if self.detached[id_] is not None else None
             return window
 
@@ -237,19 +265,29 @@ class AppWindow(QMainWindow):
                 self.input_field.setCurrentIndex(self.input_field.count() - 1)
             self.use_input()
 
-        # Define base widgets
+        def open_file_in_view(file_path: str) -> None:
+            if not Path(file_path).is_file():
+                return
+
+            if file_path.endswith(tuple(SUPPORTED_IMAGE_EXTENSIONS)):
+                self.update_image(file_path, Path(file_path).read_bytes())
+            else:
+                self.update_text(file_path, Path(file_path).read_text(encoding='utf8'))
+
+        # Define widget attributes
+        # Cannot be defined in init_objects() as walrus operators are not allowed for object attribute assignment.
+        # This works in the standard AST, but is a seemingly arbitrary limitation set by the interpreter.
+        # See: https://stackoverflow.com/questions/64055314/why-cant-pythons-walrus-operator-be-used-to-set-instance-attributes#answer-66617839
         (
             self.input_field, self.image_size_label, self.text_size_label,
             self.image_detach_button, self.text_detach_button, self.clear_picture, self.copy_picture,
-            self.clear_text, self.copy_text, get_button, scan_button,
-            self.media_frame, self.text_frame, self.media_output, self.text_output,
-            subdomain_field, root_folder_field
+            self.clear_text, self.copy_text, self.media_frame, self.text_frame,
+            self.media_output, self.text_output, self.cache_explorer
         ) = (
             HistoryComboBox(self), QLabel(self), QLabel(self),
             QPushButton(self), QPushButton(self), QPushButton(self), QPushButton(self),
-            QPushButton(self), QPushButton(self), QPushButton(self), QPushButton(self),
-            QFrame(self), QFrame(self), QGraphicsView(self), ExternalTextBrowser(self),
-            PasteLineEdit(self), PasteLineEdit(self)
+            QPushButton(self), QPushButton(self), QFrame(self), QFrame(self),
+            QGraphicsView(self), ExternalTextBrowser(self), CacheExplorer(self)
         )
 
         init_objects({
@@ -290,11 +328,11 @@ class AppWindow(QMainWindow):
                 'size': {'maximum': (160, None), 'minimum': (80, None)},
                 'clicked':  DeferredCallable(app().clipboard().setText, self.text_output.toPlainText)
             },
-            get_button: {
+            (get_button := QPushButton(self)): {
                 'size': {'maximum': (40, None)},
                 'clicked': self.use_input
             },
-            scan_button: {
+            (scan_button := QPushButton(self)): {
                 'size': {'maximum': (55, None)},
                 'clicked': DeferredCallable(self.use_input, scan=True)
             },
@@ -306,15 +344,19 @@ class AppWindow(QMainWindow):
             self.input_field.lineEdit(): {
                 'returnPressed': self.use_input
             },
-            subdomain_field: {
+            (subdomain_field := PasteLineEdit(self)): {
                 'text': app().client.sub_host, 'disabled': True,
                 'size': {'fixed': (125, None)}
             },
-            root_folder_field: {
+            (root_folder_field := PasteLineEdit(self)): {
                 'text': app().client.parent_path, 'disabled': True,
                 'size': {'fixed': (28, None)}
             },
 
+            self.cache_explorer: {
+                'openFileInView': open_file_in_view
+            },
+
             # Outputs
             self.media_output: {
                 'disabled': True, 'size': {'minimum': (None, 28)},
@@ -327,7 +369,7 @@ class AppWindow(QMainWindow):
                 'lineWrapMode': QTextEdit.LineWrapMode(app().settings['gui/text_output/line_wrap_mode']),
                 'openLinks': False, 'anchorClicked': self.navigate_to
             }
-        }, translator=app().translator)
+        })
 
         app().init_translations({
             self.change_title: 'app.name',
@@ -349,56 +391,86 @@ class AppWindow(QMainWindow):
 
         self.text_output.connect_key_to(Qt.Key_Left, prev_in_history)
         self.text_output.connect_key_to(Qt.Key_Right, next_in_history)
-
-        main_widget = QWidget()
-        layout = QGridLayout()
-        top = QHBoxLayout()
         self.outputs = QHBoxLayout()
-        media_layout = QVBoxLayout(self.media_frame)
-        media_top = QHBoxLayout()
-        media_bottom = QHBoxLayout()
-        text_layout = QVBoxLayout(self.text_frame)
-        text_top = QHBoxLayout()
-        text_bottom = QHBoxLayout()
-        bottom = QGridLayout()
-        statuses = QHBoxLayout()
 
-        self.setCentralWidget(main_widget)
-        main_widget.setLayout(layout)
-        layout.addLayout(top, 10, 0, Qt.AlignTop)
-        layout.addLayout(self.outputs, 20, 0, Qt.AlignHCenter)
-        layout.addLayout(bottom, 30, 0, Qt.AlignBottom)
+        init_layouts({
+            # Text widget layouts
+            (text_bottom := QHBoxLayout()): {
+                'items': [
+                    (self.clear_text, Qt.AlignLeft),
+                    (self.copy_text, Qt.AlignLeft)
+                ]
+            },
+            (text_top := QHBoxLayout()): {
+                'items': [
+                    (self.text_size_label, Qt.AlignLeft),
+                    (self.text_detach_button, Qt.AlignRight)
+                ]
+            },
+            QVBoxLayout(self.text_frame): {
+                'items': [
+                    text_top, self.text_output, text_bottom
+                ]
+            },
 
-        top.addWidget(subdomain_field)
-        top.addWidget(root_folder_field)
-        top.addWidget(self.input_field)
-        top.addWidget(get_button)
-        top.addWidget(scan_button)
-        top.setSpacing(2)
+            # Media widget layouts
+            (media_bottom := QHBoxLayout()): {
+                'items': [
+                    (self.clear_picture, Qt.AlignLeft),
+                    (self.copy_picture, Qt.AlignLeft)
+                ]
+            },
+            (media_top := QHBoxLayout()): {
+                'items': [
+                    (self.image_size_label, Qt.AlignLeft),
+                    (self.image_detach_button, Qt.AlignRight)
+                ]
+            },
+            QVBoxLayout(self.media_frame): {
+                'items': [
+                    media_top, self.media_output, media_bottom
+                ]
+            },
 
-        self.outputs.addWidget(self.media_frame)
-        self.outputs.addWidget(self.text_frame)
+            # Middle parent layouts
+            self.outputs: {
+                'items': [
+                    self.media_frame, self.text_frame
+                ]
+            },
+            (middle := QHBoxLayout()): {
+                'items': [
+                    (self.cache_explorer, Qt.AlignLeft),
+                    (self.outputs, Qt.AlignHCenter)
+                ]
+            },
 
-        # noinspection Duplicates
-        media_layout.addLayout(media_top)
-        media_layout.addWidget(self.media_output)
-        media_layout.addLayout(media_bottom)
-        media_top.addWidget(self.image_size_label, Qt.AlignLeft)
-        media_top.addWidget(self.image_detach_button, Qt.AlignRight)
-        media_bottom.addWidget(self.clear_picture, Qt.AlignLeft)
-        media_bottom.addWidget(self.copy_picture, Qt.AlignLeft)
+            # Top widget layout
+            (top := QHBoxLayout()): {
+                'items': [
+                    subdomain_field, root_folder_field, self.input_field,
+                    get_button, scan_button
+                ]
+            },
 
-        # noinspection Duplicates
-        text_layout.addLayout(text_top)
-        text_layout.addWidget(self.text_output)
-        text_layout.addLayout(text_bottom)
-        text_top.addWidget(self.text_size_label, Qt.AlignLeft)
-        text_top.addWidget(self.text_detach_button, Qt.AlignRight)
-        text_bottom.addWidget(self.clear_text, Qt.AlignLeft)
-        text_bottom.addWidget(self.copy_text, Qt.AlignLeft)
-        text_bottom.setSpacing(5)
+            # Main layout
+            (layout := QGridLayout()): {
+                'items': [
+                    (top, 10, 0, Qt.AlignTop),
+                    (middle, 20, 0, Qt.AlignHCenter),
+                    # (bottom, 30, 0, Qt.AlignBottom)
+                ]
+            }
+        })
 
-        bottom.addLayout(statuses, 10, 0)
+        init_objects({
+            (main_widget := QWidget()): {'layout': layout},
+
+            text_bottom: {'spacing': 5},
+            top: {'spacing': 2},
+
+            self: {'centralWidget': main_widget}
+        })
 
     def change_title(self, name: str) -> None:
         """Change the window title, includes the version number."""
@@ -412,7 +484,7 @@ class AppWindow(QMainWindow):
                 or
                 sys.platform.startswith('linux') and str_path.startswith('/')
         ):
-            webbrowser.open(str_path)
+            QDesktopServices.openUrl(QUrl(str_path))
         else:
             self.input_field.addItem(str_path)
             self.input_field.setCurrentIndex(self.input_field.count() - 1)
@@ -439,27 +511,13 @@ class AppWindow(QMainWindow):
 
         if search_path:
             if scan:
-                app().client.recursive_search(search_path)
+                app().start_worker(RecursiveSearch(app().client, search_path))
             else:
                 app().client.get_hi_data(search_path)
 
-    @staticmethod
-    def size_label_for(data: bytes) -> str:
-        """Return the best display unit to describe the given data's size.
-
-        Ex: Bytes, KiB, MiB, GiB, TiB
-        """
-        display_unit = 'Bytes'
-        for size_label in BYTE_UNITS:
-            if len(data) >= (BYTE_UNITS[size_label] // 2):
-                display_unit = size_label
-            else:
-                break
-        return display_unit
-
     def update_image(self, _: str, data: bytes) -> None:
         """Update the image output with the given data."""
-        display_unit: str = self.size_label_for(data)
+        display_unit: str = size_label_for(len(data))
         display_size: int = round(len(data) / BYTE_UNITS[display_unit], 4)
 
         self.clear_picture.setDisabled(False)
@@ -473,9 +531,10 @@ class AppWindow(QMainWindow):
         ))
         self.resize_image()
 
-    def update_text(self, search_path: str, data: dict[str, Any]) -> None:
+    def update_text(self, search_path: str, data: str | dict[str, Any]) -> None:
         """Update the text output with the given data."""
-        data = json.dumps(data, indent=2)
+        if isinstance(data, dict):
+            data = json.dumps(data, indent=2)
 
         scroll_to_top(self.text_output)
         self.clear_text.setDisabled(False)
@@ -483,7 +542,7 @@ class AppWindow(QMainWindow):
         self.text_output.setDisabled(False)
         self.text_output.clear()
 
-        display_unit: str = self.size_label_for(data.encode('utf8', errors='ignore'))
+        display_unit: str = size_label_for(len(data.encode('utf8', errors='ignore')))
         display_size: int = round(len(data) / BYTE_UNITS[display_unit], 4)
 
         # Load up to 8 MiB of text data
@@ -492,7 +551,7 @@ class AppWindow(QMainWindow):
         else:
             output = tr(
                 'gui.outputs.text.errors.too_large',
-                app().client.os_path(search_path)
+                app().client.to_os_path(search_path)
             )
 
         original_output = output
@@ -538,6 +597,7 @@ class AppWindow(QMainWindow):
         if self.current_image is not None:
             new = self.current_image.copy()
             self.media_output.scene().clear()  # Clear buffer, otherwise causes memory leak
+
             if self.current_image.size() != self.media_output.viewport().size():
                 # Create a new image from the copied source image, scaled to fit the window.
                 new = new.scaled(
@@ -545,6 +605,7 @@ class AppWindow(QMainWindow):
                     Qt.AspectRatioMode(app().settings['gui/media_output/aspect_ratio_mode']),
                     Qt.TransformationMode(app().settings['gui/media_output/transformation_mode'])
                 )
+
             # Add image to buffer
             self.media_output.scene().addPixmap(new)
 
@@ -555,10 +616,9 @@ class AppWindow(QMainWindow):
         super().show()
 
         if app().first_launch:
-            readme = ReadmeViewer()
-            readme.setWindowTitle(tr('gui.readme_viewer.title_first_launch'))
-            readme.show()
+            app().windows['readme_viewer'].show()
             app().show_dialog('information.first_launch', self)
+
         elif not self.shown_key_warning and app().client.token is None:
             app().show_dialog('warnings.empty_token', self)
             self.__class__.shown_key_warning = True
