@@ -6,10 +6,10 @@ from __future__ import annotations
 
 __all__ = (
     'ExceptionReporter',
+    'format_tb',
 )
 
 import traceback
-import webbrowser
 from getpass import getuser
 from types import TracebackType
 
@@ -19,10 +19,11 @@ from PySide6.QtWidgets import *
 
 from ..._version import __version__
 from ...events import EventBus
-from ...exceptions import ExceptionEvent
+from ...exception_hook import ExceptionEvent
 from ...models import DeferredCallable
 from ...models import DistributedCallable
 from ...utils.gui import delete_layout_widgets
+from ...utils.gui import init_layouts
 from ...utils.gui import init_objects
 from ...utils.network import encode_url_params
 from ..app import app
@@ -36,7 +37,6 @@ def format_tb(tb: TracebackType) -> str:
     return '\n'.join(traceback.format_tb(tb))
 
 
-# noinspection PyArgumentList
 class ExceptionReporter(QWidget):
     """A :py:class:`QWidget` that displays logged exceptions and their traceback."""
 
@@ -52,13 +52,9 @@ class ExceptionReporter(QWidget):
         self._init_ui()
 
     def _init_ui(self) -> None:
-        layout = QHBoxLayout(self)
         self.left_panel = QFrame(self)
         self.right_panel = QFrame(self)
 
-        left_label = QLabel(self.left_panel)
-        right_label = QLabel(self.right_panel)
-        clear_all_button = QPushButton(self.left_panel)
         self.trace_back_viewer = ExternalTextBrowser(self)
         self.clear_button = QPushButton(self.right_panel)
         self.report_button = QPushButton(self.right_panel)
@@ -68,25 +64,34 @@ class ExceptionReporter(QWidget):
         self.scroll_widget = QWidget(self.scroll_area)
         self.scroll_widget.setLayout(QVBoxLayout())
         self.scroll_widget.layout().setAlignment(Qt.AlignTop)
-        self.scroll_area.setWidget(self.scroll_widget)
 
         init_objects({
             self.scroll_area: {
+                'widget': self.scroll_widget,
                 'widgetResizable': True
             },
-            clear_all_button: {
+
+            self.left_panel: {'layout': QVBoxLayout()},
+            self.right_panel: {'layout': QVBoxLayout()},
+            (left_label := QLabel(self.left_panel)): {},
+            (right_label := QLabel(self.right_panel)): {},
+
+            (clear_all_button := QPushButton(self.left_panel)): {
                 'clicked': self.clear_all_exceptions
             },
+
             self.trace_back_viewer: {
                 'disabled': True,
                 'font': QFont('consolas', 10),
                 'lineWrapMode': QTextEdit.NoWrap
             },
+
             self.clear_button: {
                 'disabled': True,
                 'size': {'maximum': (self.report_button.width() // 1, None)},
                 'clicked': self.clear_current_exception
             },
+
             self.report_button: {
                 'disabled': True,
                 'clicked': self.report_current_exception
@@ -106,28 +111,34 @@ class ExceptionReporter(QWidget):
             self.report_button.setText: 'gui.exception_reporter.report'
         })
 
-        layout.addWidget(self.left_panel)
-        layout.addWidget(self.right_panel)
+        init_layouts({
+            # Left frame layout
+            self.left_panel.layout(): {
+                'items': [left_label, self.scroll_area, clear_all_button]
+            },
+
+            # Right frame layouts
+            (buttons := QHBoxLayout()): {
+                'items': [self.report_button, self.clear_button]
+            },
+            self.right_panel.layout(): {
+                'items': [right_label, self.trace_back_viewer, buttons]
+            },
+
+            # Main layout
+            QHBoxLayout(self): {
+                'items': [self.left_panel, self.right_panel]
+            }
+        })
+
         self.left_panel.setLayout(QVBoxLayout())
         self.right_panel.setLayout(QVBoxLayout())
-        buttons = QHBoxLayout()
-
-        self.left_panel.layout().addWidget(left_label)
-        self.left_panel.layout().addWidget(self.scroll_area)
-        self.left_panel.layout().addWidget(clear_all_button)
-
-        self.right_panel.layout().addWidget(right_label)
-        self.right_panel.layout().addWidget(self.trace_back_viewer)
-        self.right_panel.layout().addLayout(buttons)
-        buttons.addWidget(self.report_button)
-        buttons.addWidget(self.clear_button)
 
     def report_current_exception(self) -> None:
         """Report the current exception to the exception logger."""
         exc_name: str = type(self.logger.exception_log[self.selected].exception).__name__
         exc_msg:  str = str(self.logger.exception_log[self.selected].exception).rstrip(".")
-        exc_tb:   str = format_tb(self.logger.exception_log[self.selected].traceback).strip()
-        exc_tb.replace(getuser(), '%USERNAME%')  # Hide the username in the traceback
+        exc_tb:   str = format_tb(self.logger.exception_log[self.selected].traceback).strip().replace(getuser(), '%USERNAME%')
         base:     str = 'https://github.com/Cubicpath/HaloInfiniteGetter/issues/new'
         params:   dict[str, str] = {
             'template': 'bug-report.yaml',
@@ -138,7 +149,7 @@ class ExceptionReporter(QWidget):
             'version': f'v{__version__}',
             'logs': f'{exc_name}: {exc_msg}\n\n{exc_tb}'
         }
-        webbrowser.open(f'{base}?{encode_url_params(params)}')
+        QDesktopServices.openUrl(QUrl(f'{base}?{encode_url_params(params)}'))
 
     def disable_exception_widgets(self) -> None:
         """Disables the widgets on the right pane."""
