@@ -16,12 +16,13 @@ from PySide6.QtGui import *
 from PySide6.QtWidgets import *
 
 from ...constants import *
+from ...utils.gui import init_layouts
+from ...utils.gui import init_objects
 from ..app import app
 from ..app import tr
 from ..widgets import ExternalTextBrowser
 
 
-# noinspection PyArgumentList
 class ReadmeViewer(QWidget):
     """Widget that formats and shows the project's README.md, stored in the projects 'Description' metadata tag."""
     README_TEXT: Final[str] = metadata(HI_PACKAGE_NAME)['Description']
@@ -34,29 +35,33 @@ class ReadmeViewer(QWidget):
         self.readme_viewer: ExternalTextBrowser
         self._init_ui()
 
-    def _dummy_func(self) -> None:
-        """Must exist otherwise ReadmeViewer instances will be garbage collected through Context Menu deletion. Don't ask, just accept."""
-
     def _init_ui(self) -> None:
         self.readme_viewer = ExternalTextBrowser(self)
-        close_button = QPushButton("Close", self, clicked=self.close)
 
-        self.readme_viewer.connect_key_to(Qt.Key_Any, self._dummy_func)  # Refer to self._dummy_func
+        init_objects({
+            (close_button := QPushButton(self)): {
+                'size': {'minimum': (None, 40)},
+                'font': QFont(close_button.font().family(), 16),
+                'clicked': self.close
+            },
 
-        layout = QVBoxLayout(self)
-        self.setLayout(layout)
+            self.readme_viewer: {
+                'font': QFont(self.readme_viewer.font().family(), 10),
+                'openExternalLinks': True
+            }
+        })
 
-        layout.addWidget(self.readme_viewer)
-        layout.addWidget(close_button)
+        app().init_translations({
+            self.setWindowTitle: 'gui.readme_viewer.title',
+
+            close_button.setText: 'gui.readme_viewer.close'
+        })
+
+        init_layouts({
+            # Main layout
+            QVBoxLayout(self): {
+                'items': [self.readme_viewer, close_button]
+            }
+        })
 
-        self.readme_viewer.setOpenExternalLinks(True)
         self.readme_viewer.set_hot_reloadable_text(self.README_TEXT, 'markdown')
-        self.readme_viewer.setFont(QFont(self.readme_viewer.font().family(), 10))
-        close_button.setMinimumHeight(40)
-        close_button.setFont(QFont(close_button.font().family(), 16))
-
-    def closeEvent(self, event: QCloseEvent) -> None:
-        """Manually signal the readme_viewer for garbage collection."""
-        super().closeEvent(event)
-        self.readme_viewer.deleteLater()
-        event.accept()
