@@ -8,7 +8,6 @@ __all__ = (
     'SettingsWindow',
 )
 
-import webbrowser
 from pathlib import Path
 
 from PySide6.QtCore import *
@@ -18,29 +17,35 @@ from PySide6.QtWidgets import *
 from ...constants import *
 from ...events import EventBus
 from ...models import DeferredCallable
+from ...models import Singleton
 from ...tomlfile import TomlEvents
+from ...utils.gui import init_layouts
 from ...utils.gui import init_objects
 from ..app import app
 from ..app import tr
 from ..widgets import PasteLineEdit
-from .application import AppWindow
+from ..widgets import TranslatableComboBox
 
 
-# noinspection PyArgumentList
-class SettingsWindow(QWidget):
-    """Window that provides user interaction with the application's settings."""
+# noinspection PyTypeChecker
+class SettingsWindow(Singleton, QWidget):
+    """Window that provides user interaction with the application's settings.
 
-    def __init__(self, parent: AppWindow, size: QSize) -> None:
-        """Create a new settings window. Should only have one instance."""
+    :py:class:`SettingsWindow` is a singleton and can be accessed via the class using the SettingsWindow.instance() class method.
+    """
+    _singleton_base_type = QWidget
+    _singleton_check_ref = False
+
+    def __init__(self, size: QSize) -> None:
+        """Create the settings window."""
         super().__init__()
-        self.app_window = parent
 
         self.setWindowTitle(tr('gui.settings.title'))
         self.setWindowIcon(app().icon_store['settings'])
         self.resize(size)
         self.setFixedWidth(self.width())
 
-        # Create a tr DeferredCallable with every tuple acting as arguments.
+        # Show an error dialog on import failure
         EventBus['settings'].subscribe(
             DeferredCallable(app().show_dialog, 'errors.settings.import_failure', self, description_args=(app().settings.path,)),
             TomlEvents.Fail, event_predicate=lambda event: event.failure == 'import'
@@ -58,16 +63,16 @@ class SettingsWindow(QWidget):
 
         def import_settings() -> None:
             """Import settings from a chosen TOML file."""
-            file_path = Path(QFileDialog.getOpenFileName(self, tr('gui.settings.import'),
-                                                         str(HI_CONFIG_PATH), 'TOML Files (*.toml);;All files (*.*)')[0])
+            file_path = Path(QFileDialog.getOpenFileName(self, caption=tr('gui.settings.import'),
+                                                         dir=str(HI_CONFIG_PATH), filter='TOML Files (*.toml);;All files (*.*)')[0])
             if file_path.is_file():
                 if app().settings.import_from(file_path):
                     save_button.setDisabled(False)
 
         def export_settings() -> None:
             """Export current settings to a chosen file location."""
-            file_path = Path(QFileDialog.getSaveFileName(self, tr('gui.settings.export'),
-                                                         str(HI_CONFIG_PATH), 'TOML Files (*.toml);;All files (*.*)')[0])
+            file_path = Path(QFileDialog.getSaveFileName(self, caption=tr('gui.settings.export'),
+                                                         dir=str(HI_CONFIG_PATH), filter='TOML Files (*.toml);;All files (*.*)')[0])
             if str(file_path) != '.':
                 app().settings.export_to(file_path)
 
@@ -102,68 +107,61 @@ class SettingsWindow(QWidget):
             del app().client.token
             self.token_clear_button.setDisabled(True)
 
-        # Define base widgets
+        # Define widget attributes
+        # Cannot be defined in init_objects() as walrus operators are not allowed for object attribute assignment.
+        # This works in the standard AST, but is a seemingly arbitrary limitation set by the interpreter.
+        # See: https://stackoverflow.com/questions/64055314/why-cant-pythons-walrus-operator-be-used-to-set-instance-attributes#answer-66617839
         (
-            theme_label, aspect_ratio_label, transformation_label, line_wrap_label,
-            save_button, reload_button, import_button, export_button, open_editor_button,
-            key_show_button, key_copy_button, self.key_set_button, self.token_clear_button,
-            self.theme_dropdown, self.aspect_ratio_dropdown, self.transformation_dropdown, self.line_wrap_dropdown,
+            self.key_set_button, self.token_clear_button,
+            self.theme_dropdown, self.aspect_ratio_dropdown, self.transformation_dropdown, self.line_wrap_dropdown, self.icon_mode_dropdown,
             self.key_field
         ) = (
-            QLabel(self), QLabel(self), QLabel(self), QLabel(self),
-            QPushButton(self), QPushButton(self), QPushButton(self), QPushButton(self), QPushButton(self),
-            QPushButton(self), QPushButton(self), QPushButton(self), QPushButton(self),
-            QComboBox(self), QComboBox(self), QComboBox(self), QComboBox(self),
+            QPushButton(self), QPushButton(self),
+            TranslatableComboBox(self), TranslatableComboBox(self), TranslatableComboBox(self), TranslatableComboBox(self), TranslatableComboBox(self),
             PasteLineEdit(self)
         )
 
-        for subscribe_params in (
-                (DeferredCallable(self.app_window.resize_image), TomlEvents.Set, lambda event: event.key.startswith('gui/media_output/')),
-                (lambda val: self.app_window.text_output.setLineWrapMode(val.new), TomlEvents.Set, lambda event: event.key == 'gui/text_output/line_wrap_mode'),
-                (DeferredCallable(save_button.setDisabled, False), TomlEvents.Set, lambda event: event.old != event.new),
-                (DeferredCallable(save_button.setDisabled, True), TomlEvents.Export, lambda event: event.toml_file.path == event.path),
-                (DeferredCallable(save_button.setDisabled, True), TomlEvents.Import, lambda event: event.toml_file.path == event.path),
-                (DeferredCallable(self.refresh_dropdowns), TomlEvents.Import)
-        ): EventBus['settings'].subscribe(*subscribe_params)
-
         init_objects({
             # Labels
-            theme_label: {
+            (theme_label := QLabel(self)): {
                 'size': {'maximum': (85, None)}
             },
-            aspect_ratio_label: {
+            (aspect_ratio_label := QLabel(self)): {
                 'size': {'maximum': (90, None)}
             },
-            transformation_label: {
+            (transformation_label := QLabel(self)): {
                 'size': {'maximum': (90, None)}
             },
-            line_wrap_label: {
+            (line_wrap_label := QLabel(self)): {
+                'size': {'maximum': (90, None)}
+            },
+            (icon_mode_label := QLabel(self)): {
                 'size': {'maximum': (90, None)}
             },
 
             # Buttons
-            save_button: {
+            (save_button := QPushButton(self)): {
                 'disabled': True,
                 'size': {'maximum': (50, None)},
                 'clicked': app().settings.save
             },
-            reload_button: {
+            (reload_button := QPushButton(self)): {
                 'size': {'maximum': (60, None)},
                 'clicked': app().settings.reload
             },
-            import_button: {
+            (import_button := QPushButton(self)): {
                 'clicked': import_settings
             },
-            export_button: {
+            (export_button := QPushButton(self)): {
                 'clicked': export_settings
             },
-            open_editor_button: {
-                'clicked': DeferredCallable(webbrowser.open, lambda: app().settings.path)
+            (open_editor_button := QPushButton(self)): {
+                'clicked': DeferredCallable(QDesktopServices.openUrl, lambda: QUrl(app().settings.path.as_uri()))
             },
-            key_show_button: {
+            (key_show_button := QPushButton(self)): {
                 'clicked': toggle_key_visibility
             },
-            key_copy_button: {
+            (key_copy_button := QPushButton(self)): {
                 'clicked': DeferredCallable(app().clipboard().setText, lambda: app().client.wpauth)
             },
             self.key_set_button: {
@@ -226,17 +224,34 @@ class SettingsWindow(QWidget):
                     'gui.settings.text.line_wrap.fixed_pixel',
                     'gui.settings.text.line_wrap.fixed_column'
                 )
+            },
+            self.icon_mode_dropdown: {
+                'activated': DeferredCallable(
+                    app().settings.__setitem__,
+                    'gui/cache_explorer/icon_mode',
+                    self.icon_mode_dropdown.currentIndex
+                ),
+                'items': (
+                    'gui.settings.cache.icon_mode.no_icons',
+                    'gui.settings.cache.icon_mode.default',
+                    'gui.settings.cache.icon_mode.preview'
+                )
             }
-        }, translator=tr)
+        })
 
         app().init_translations({
             self.setWindowTitle: 'gui.settings.title',
+            self.aspect_ratio_dropdown.translate_items: '',
+            self.line_wrap_dropdown.translate_items: '',
+            self.theme_dropdown.translate_items: '',
+            self.transformation_dropdown.translate_items: '',
 
             # Labels
             theme_label.setText: 'gui.settings.theme',
             aspect_ratio_label.setText: 'gui.settings.media.aspect_ratio',
             transformation_label.setText: 'gui.settings.media.image_transform',
             line_wrap_label.setText: 'gui.settings.text.line_wrap',
+            icon_mode_label.setText: 'gui.settings.cache.icon_mode',
 
             # Buttons
             save_button.setText: 'gui.settings.save',
@@ -250,53 +265,66 @@ class SettingsWindow(QWidget):
             self.token_clear_button.setText: 'gui.settings.auth.clear_token'
         })
 
-        # Define layouts
-        layout = QGridLayout(self)  # Main layout
-        top = QHBoxLayout()
-        middle = QVBoxLayout()
-        theme_layout = QHBoxLayout()
-        output_layout = QGridLayout()
-        bottom = QVBoxLayout()
-        key_layout = QHBoxLayout()
-        token_layout = QHBoxLayout()
+        init_layouts({
+            # Add bottom widgets
+            (token_layout := QHBoxLayout()): {
+                'items': [self.token_clear_button]
+            },
+            (key_layout := QHBoxLayout()): {
+                'items': [key_copy_button, self.key_field, self.key_set_button]
+            },
+            (bottom := QVBoxLayout()): {
+                'items': [key_show_button, key_layout, token_layout]
+            },
 
-        # Assign positions of layouts
-        layout.addLayout(top, 0, 0, Qt.AlignTop)
-        layout.addLayout(middle, 10, 0, Qt.AlignTop)
-        layout.addLayout(bottom, 20, 0, Qt.AlignBottom)
+            # Add middle widgets
+            (cache_layout := QHBoxLayout()): {
+                'items': [icon_mode_label, self.icon_mode_dropdown]
+            },
+            (output_layout := QGridLayout()): {
+                'items': [
+                    (aspect_ratio_label, 0, 0),
+                    (self.aspect_ratio_dropdown, 0, 10),
+                    (transformation_label, 0, 20),
+                    (self.transformation_dropdown, 0, 30),
+                    (line_wrap_label, 10, 0),
+                    (self.line_wrap_dropdown, 10, 10)
+                ]
+            },
+            (theme_layout := QHBoxLayout()): {
+                'items': [theme_label, self.theme_dropdown]
+            },
+            (middle := QVBoxLayout()): {
+                'items': [open_editor_button, theme_layout, output_layout, cache_layout]
+            },
 
-        # Add top widgets
-        top.addWidget(save_button)
-        top.addWidget(reload_button)
-        top.addWidget(import_button)
-        top.addWidget(export_button)
+            # Add top widgets
+            (top := QHBoxLayout()): {
+                'items': [save_button, reload_button, import_button, export_button]
+            },
 
-        # Add middle widgets
-        middle.addWidget(open_editor_button)
-        middle.addLayout(theme_layout)
-        middle.addLayout(output_layout)
-        theme_layout.addWidget(theme_label)
-        theme_layout.addWidget(self.theme_dropdown)
-        output_layout.addWidget(aspect_ratio_label, 0, 0)
-        output_layout.addWidget(self.aspect_ratio_dropdown, 0, 10)
-        output_layout.addWidget(transformation_label, 0, 20)
-        output_layout.addWidget(self.transformation_dropdown, 0, 30)
-        output_layout.addWidget(line_wrap_label, 10, 0)
-        output_layout.addWidget(self.line_wrap_dropdown, 10, 10)
+            # Main layout
+            QGridLayout(self): {
+                'items': [
+                    (top, 0, 0, Qt.AlignTop),
+                    (middle, 10, 0, Qt.AlignTop),
+                    (bottom, 20, 0, Qt.AlignBottom)
+                ]
+            }
+        })
 
-        # Add bottom widgets
-        bottom.addWidget(key_show_button)
-        bottom.addLayout(key_layout)
-        bottom.addLayout(token_layout)
-        key_layout.addWidget(key_copy_button)
-        key_layout.addWidget(self.key_field)
-        key_layout.addWidget(self.key_set_button)
-        token_layout.addWidget(self.token_clear_button)
+        for subscribe_params in (
+                (DeferredCallable(save_button.setDisabled, False), TomlEvents.Set, lambda event: event.old != event.new),
+                (DeferredCallable(save_button.setDisabled, True), TomlEvents.Export, lambda event: event.toml_file.path == event.path),
+                (DeferredCallable(save_button.setDisabled, True), TomlEvents.Import, lambda event: event.toml_file.path == event.path),
+                (DeferredCallable(self.refresh_dropdowns), TomlEvents.Import)
+        ): EventBus['settings'].subscribe(*subscribe_params)
 
         self.refresh_dropdowns()
 
     def refresh_dropdowns(self) -> None:
         """Refresh all dropdown widgets with the current settings assigned to them."""
+        self.icon_mode_dropdown.setCurrentIndex(app().settings['gui/cache_explorer/icon_mode'])
         self.aspect_ratio_dropdown.setCurrentIndex(app().settings['gui/media_output/aspect_ratio_mode'])
         self.transformation_dropdown.setCurrentIndex(app().settings['gui/media_output/transformation_mode'])
         self.line_wrap_dropdown.setCurrentIndex(app().settings['gui/text_output/line_wrap_mode'])
@@ -307,9 +335,15 @@ class SettingsWindow(QWidget):
     def showEvent(self, event: QShowEvent) -> None:
         """Auto hides the key upon un-minimizing."""
         super().showEvent(event)
-        self.key_set_button.setDisabled(True)
-        self.key_field.setAlignment(Qt.AlignCenter)
-        self.key_field.setDisabled(True)
-        self.key_field.setText(app().client.hidden_key())
-        self.token_clear_button.setDisabled(not app().client.token)
+
+        init_objects({
+            self.key_field: {
+                'disabled': True,
+                'alignment': Qt.AlignCenter,
+                'text': app().client.hidden_key()
+            },
+            self.key_set_button: {'disabled': True},
+            self.token_clear_button: {'disabled': not app().client.token}
+        })
+
         event.accept()
