@@ -7,33 +7,32 @@ from __future__ import annotations
 __all__ = (
     'Client',
     'TOKEN_PATH',
+    'WEB_DUMP_PATH',
     'WPAUTH_PATH',
 )
 
 import json
 import os
-import random
-import time
 from pathlib import Path
 from typing import Any
 from typing import Final
 
 from PySide6.QtCore import *
 from PySide6.QtNetwork import *
-from PySide6.QtWidgets import *
 
 from ..constants import *
+from ..models import CaseInsensitiveDict
 from ..utils.common import dump_data
-from ..utils.common import unique_values
 from ..utils.network import decode_url
 from ..utils.network import guess_json_utf
 from ..utils.network import is_error_status
+from ..utils.network import wait_for_reply
 from ..utils.system import hide_windows_file
 from .manager import NetworkSession
-from .structures import CaseInsensitiveDict
 
-TOKEN_PATH:  Final[Path] = HI_CONFIG_PATH / '.token'
-WPAUTH_PATH: Final[Path] = HI_CONFIG_PATH / '.wpauth'
+WEB_DUMP_PATH: Final[Path] = HI_CACHE_PATH / 'cached_requests'
+TOKEN_PATH:    Final[Path] = HI_CONFIG_PATH / '.token'
+WPAUTH_PATH:   Final[Path] = HI_CONFIG_PATH / '.wpauth'
 
 
 class Client(QObject):
@@ -86,7 +85,7 @@ class Client(QObject):
             'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:100.0) Gecko/20100101 Firefox/100.0',
         })
         self.web_session: NetworkSession = NetworkSession(self)
-        self.web_session.headers = self.api_session.headers.copy() | {
+        self.web_session.headers = self.api_session.headers | {
             'Accept': ','.join(('text/html', 'application/xhtml+xml', 'application/xml;q=0.9', 'image/avif', 'image/webp', '*/*;q=0.8')),
             'Host': self.WEB_HOST,
             'Sec-Fetch-Dest': 'document',
@@ -112,8 +111,7 @@ class Client(QObject):
         :param kwargs: Key word arguments to pass to the requests GET Request.
         """
         reply: QNetworkReply = self.api_session.get(self.api_root + path.strip(), **kwargs)
-        while not reply.isFinished():
-            QApplication.processEvents()
+        wait_for_reply(reply)
 
         # None is returned if the request was aborted
         status_code: int | None = reply.attribute(QNetworkRequest.HttpStatusCodeAttribute)
@@ -125,13 +123,13 @@ class Client(QObject):
 
         return reply
 
-    def get_hi_data(self, path: str, dump_path: Path = HI_CACHE_PATH, micro_sleep: bool = True) -> dict[str, Any] | bytes | int:
+    def get_hi_data(self, path: str, dump_path: Path = WEB_DUMP_PATH) -> dict[str, Any] | bytes | int:
         """Returns data from a path. Return type depends on the resource.
 
         :return: dict for JSON objects, bytes for media, int for error codes.
         :raises ValueError: If the response is not JSON or a supported image type.
         """
-        os_path: Path = self.os_path(path, parent=dump_path)
+        os_path: Path = self.to_os_path(path, parent=dump_path)
         data: dict[str, Any] | bytes
 
         if not os_path.is_file():
@@ -157,8 +155,6 @@ class Client(QObject):
             dump_data(os_path, data)
 
             reply.deleteLater()
-            if micro_sleep:
-                time.sleep(random.randint(100, 200) / 750)
 
         else:
             print(path)
@@ -178,31 +174,15 @@ class Client(QObject):
             return f'{key[:3]}{"." * 50}{key[-3:]}'
         return 'None'
 
-    def os_path(self, path: str, parent: Path = HI_CACHE_PATH) -> Path:
+    def to_os_path(self, path: str, parent: Path = WEB_DUMP_PATH) -> Path:
         """Translate a given GET path to the equivalent cache location."""
         return parent / self.sub_host.replace('-', '_') / self.parent_path.strip('/') / path.replace('/file/', '/').lower()
 
-    def recursive_search(self, search_path: str) -> None:
-        """Recursively get Halo Waypoint files linked to the search_path through Mapping keys."""
-        if self.searched_paths.get(search_path, 0) >= 2:
-            return
-
-        data: dict[str, Any] | bytes | int = self.get_hi_data(search_path)
-        if isinstance(data, (bytes, int)):
-            return
-
-        for value in unique_values(data):
-            if isinstance(value, str):
-                if '/' not in value:
-                    continue
-
-                end = value.split('.')[-1].lower()
-                if end in ('json',):
-                    path = 'progression/file/' + value
-                    self.searched_paths.update({path: self.searched_paths.get(path, 0) + 1})
-                    self.recursive_search(path)
-                elif end in SUPPORTED_IMAGE_EXTENSIONS:
-                    self.get_hi_data('images/file/' + value)
+    def to_get_path(self, path: str) -> str:
+        """Translate a given cache location to the equivalent GET path."""
+        resource = path.split(self.parent_path, maxsplit=1)[1]
+        pre, post = resource.split("/", maxsplit=1)
+        return f'{pre}/file/{post}'
 
     def refresh_auth(self) -> None:
         """Refreshes authentication to Halo Waypoint servers.
@@ -210,10 +190,7 @@ class Client(QObject):
         wpauth MUST have a value for this to work. A lone 343 spartan token is not enough to generate a new one.
         """
         reply: QNetworkReply = self.web_session.get('https://www.halowaypoint.com/')
-
-        # TODO: After moving client to separate thread, remove this and QtWidgets import.
-        while not reply.isFinished():
-            QApplication.processEvents()
+        wait_for_reply(reply)
 
         wpauth: str = decode_url(self.web_session.cookies.get('wpauth') or '')
         token:  str = decode_url(self.web_session.cookies.get('343-spartan-token') or '')
@@ -246,6 +223,7 @@ class Client(QObject):
         self._token = value[key_start_index:].rstrip()
         self.set_cookie('343-spartan-token', self._token)
         self.api_session.headers['x-343-authorization-spartan'] = self._token
+
         hide_windows_file(TOKEN_PATH, unhide=True)
         TOKEN_PATH.write_text(self._token, encoding='utf8')
         hide_windows_file(TOKEN_PATH)
@@ -269,6 +247,7 @@ class Client(QObject):
         value = decode_url(value)
         self._wpauth = value.split(':')[-1].strip()
         self.set_cookie('wpauth', self._wpauth)
+
         hide_windows_file(WPAUTH_PATH, unhide=True)
         WPAUTH_PATH.write_text(self._wpauth, encoding='utf8')
         hide_windows_file(WPAUTH_PATH)
