@@ -22,8 +22,7 @@ from typing import Any
 from typing import Final
 from typing import TypeAlias
 
-import toml.decoder
-import toml.encoder
+import toml
 from toml.decoder import CommentValue
 
 from .events import *
@@ -57,7 +56,7 @@ class TomlEvents:
 
     class File(TomlEvent):
         """Accessing a File on disk."""
-        __slots__ = ('toml_file', 'path')
+        __slots__ = ('path',)
 
         def __init__(self, toml_file: TomlFile, path: Path) -> None:
             super().__init__(toml_file=toml_file)
@@ -65,15 +64,15 @@ class TomlEvents:
 
     class Import(File):
         """Loading a TomlFile."""
-        __slots__ = ('toml_file', 'path')
+        __slots__ = ()
 
     class Export(File):
         """Exporting a TomlFile to disk"""
-        __slots__ = ('toml_file', 'path')
+        __slots__ = ()
 
     class KeyAccess(TomlEvent):
         """A key's value is accessed."""
-        __slots__ = ('toml_file', 'key')
+        __slots__ = ('key',)
 
         def __init__(self, toml_file: TomlFile, key: str) -> None:
             super().__init__(toml_file=toml_file)
@@ -81,7 +80,7 @@ class TomlEvents:
 
     class Get(KeyAccess):
         """Value is given."""
-        __slots__ = ('toml_file', 'key', 'value')
+        __slots__ = ('value',)
 
         def __init__(self, toml_file: TomlFile, key: str, value: TomlValue) -> None:
             super().__init__(toml_file=toml_file, key=key)
@@ -89,7 +88,7 @@ class TomlEvents:
 
     class Set(KeyAccess):
         """Value is set."""
-        __slots__ = ('toml_file', 'key', 'old', 'new')
+        __slots__ = ('old', 'new')
 
         def __init__(self, toml_file: TomlFile, key: str, old: TomlValue, new: TomlValue) -> None:
             super().__init__(toml_file=toml_file, key=key)
@@ -98,7 +97,7 @@ class TomlEvents:
 
     class Fail(TomlEvent):
         """General Failure."""
-        __slots__ = ('toml_file', 'failure',)
+        __slots__ = ('failure',)
 
         def __init__(self, toml_file: TomlFile, failure: str) -> None:
             super().__init__(toml_file=toml_file)
@@ -183,6 +182,7 @@ class TomlFile:
         if not path:
             raise ValueError('Path cannot be an empty string.')
 
+        key:   str = path
         scope: dict = self._data
         paths: list[str] = path.split('/')
 
@@ -198,10 +198,7 @@ class TomlFile:
                     elif isinstance(scope.get(key), dict):
                         scope = scope[key]
                         continue
-        else:
-            key = path
 
-        # noinspection PyUnboundLocalVariable
         return scope, key
 
     @property
